/*
* A representation of the uninitialized request state '0'.
 */
ServerAO.UNITITIALIZED = 0;

/*
 * A representation of the setup but not sent request state '1'.
 */
ServerAO.SETUP = 1;

/*
 * A representation of the sent but not having received request state '2'.
 */
ServerAO.SENT = 2;

/*
 * A representation of the in process request state '3'.
 */
ServerAO.IN_PROCESS = 3;

/*
 * A representation of the complete request state '4'.
 */
ServerAO.COMPLETE = 4;


/*
 * A representation of the HTTP status code for file retrieval on FirefFox'0'.  For other HTTP status codes go to www.w3.org/Protocols/rfc2616/rfc2616-sec10.html.
 */
ServerAO.FireFox_HTTP_File_Access = 0;

/*
 * A representation of the HTTP status code for file retrieval on OSX 'undefined'.  For other HTTP status codes go to www.w3.org/Protocols/rfc2616/rfc2616-sec10.html.
 */
ServerAO.OSX_HTTP_File_Access = null;

/*
 * A representation of the HTTP status code for OK '200'.  For other HTTP status codes go to www.w3.org/Protocols/rfc2616/rfc2616-sec10.html.
 */
ServerAO.HTTP_OK = 200;

/*
 * A representation of the HTTP status code for Bad Request '400'.  For other HTTP status codes go to www.w3.org/Protocols/rfc2616/rfc2616-sec10.html.
 */
ServerAO.HTTP_BADREQUEST = 400;

/*
 * A representation of the HTTP status code for Version Not Supported '505'.  For other HTTP status codes go to www.w3.org/Protocols/rfc2616/rfc2616-sec10.html.
 */
ServerAO.HTTP_VERSION_NOT_SUPPORTED = 505;



/*
*Constructor for a new ServerAO instance. The purpose of the Server Access Object is to 
*  create all requests for data from the server as well as requests for posting of data
*  to the server and make all preparations required to successfully make 
*  these calls.  All calls made are asynchronous.
*  parameter 1: URL - the URL of the main access point for the application
*/
function ServerAO(URL){
      this.URL = URL;

      /*
      * Executes the call to the server to post or get data. Creation and use of an XMLHttpRequest object happens within this method.
      * parameter 1: {String} callType - GET, POST, or other HTTP type
      * parameter 2: {Object} whenDone - the ViewControlObject to activate when data returns from the server.
      * parameter 3: {String} dataType - values are either Text or XML
      * parameter 4: {boolean} refresh - a indicating if refreshing should be enforced even if the data is currently cached.
      * parameter 5: {String} parameterSequence - the parameter list to be added to the URL
      * parameter 6: {String or DOM element} data - any data to be sent with a POST type request
      * parameter 7: {Object} HTTPHeaders - a map of headers and values
      */
      this.makeCall = function(callType, whenDone, dataType, refresh, parameterSequence, data, HTTPHeaders){
            if(dataType == null){
                  dataType = 'Text';//set default value
            }
            if(refresh == null){
                  refresh = false;//set default value
            }
            //get a request object for all types of browsers.
            if (!window.XMLHttpRequest) {
            /*
            *     if we are running IE 6 or older then the XMLHttpRequest method is 
            *     not defined.  If we define it the first time it is used as being 
            *     the Microsoft AactiveX method of retrieving a request object then 
            *     it will be used each time a new call is made and not need to be 
            *     defined over again.
            */
               window.XMLHttpRequest = function() {
                     return new ActiveXObject("Microsoft.XMLHTTP");
               };
            } 
            var http = null;
            try {
                  http = new XMLHttpRequest();
            } 
            catch(e) {
                  return;
            }
            var isAsynch = true;//enforce always asynchronous
            //open a connection to the server located at 'this.URL'.
            http.open(callType,this.URL+'?'+parameterSequence, isAsynch);
            if(refresh){
            //   if we are to disable caching and force a call to the server then the 'If-Modified-Since' header will need to be set to some time in the past.
              http.setRequestHeader( "If-Modified-Since", "Sat, 1 Jan 2000 0:00:00 GMT" );
            }
            //if there are headers then add them to the request
            if(this.HTTPHeaders != null){
                  for(var key in HTTPHeaders){
http.setRequestHeader(key, HTTPHeaders[key]);
                  }
            }
            /*
            *     Set the function to be called when the server communicates with the client.
            *     For the ServerAO this is an inline function that is the callback function defined below.
            *     The reason for the inline function is so that variables from within the makeCall method can be used.  If a non-inline 
            *     function is used then only the name of the function to be called is set as the onreadystatechange 
            *     method of the HTTP class.
            */
            http.onreadystatechange = function(){
                              /*
                              *     Only handle completed, finished messages from the server
                              */
                              if(http.readyState == ServerAO.COMPLETE){
                                 /*
                                    *  Retrieve the data if the server returns that the processing of the request was successful or if
                                    *  the request was directly for a file on the server disk.
                                    */
                                    var resultData = null;//null represents an error in processing
                                    if(http.status == ServerAO.HTTP_OK || http.status == ServerAO.FireFox_HTTP_File_Access
                                           || http.status == ServerAO.OSX_HTTP_File_Access){
                                           
                                          resultData = http['response'+dataType];//get the data as either Text or XML
                                    }
                                    if(whenDone != null){
                                          //instantiate the whenDone Object and call notify
                                          whenDone.notify(resultData);
                                    }
                                    http = null;//force a release of the memory for the request object
                              }

                        };
            /*
            *     Send off the completed request and include any data for a 'POST' request.
            */
            http.send(this.data);
      }
}
