/*
*	This is a history tracking tool that works for Firefox, Safari, and IE.
*
*	In Safari the use of the # anchor indicator and the hidden iframe forces a full reload of the page each time the back or forward button is used.  While 
*	this is not efficient it does work.  Safari uses a thread to update the URL so a time out is included to allow for the delay as this happens.
*
*	IE requires that the hidden iframe be included in the source of the main page in order to generate the history elements for the back and forward buttons.
*	IE 6 will return to the main page if a link takes the user off the site.
*
*	Firefox on Windows will generate a history element for each change of the URL as well as each change of the source for the hidden iframe.  Because of this 
*	no changes to the iframe are made in Firefox.  No such doubling happens in Firefox on OS X.
*/

//the milliseconds between checks for updating the page
var pageUpdateDelayMillis = 100;
//the URL of the main page
var baseURL = window.location.href.split('#')[0];



/* a method that gets called from the onload of the main page.
*/
function loadFirstSubPage(aCmd){
	//retreive an existing command if one exists
    aCmd = getSubPageCommand() || aCmd;
    //if a cmmand was passed in or found update the URL with the anchor indicator
    if(aCmd){
        updateURL(aCmd, null);
    }
    //since url update is on a seperate thread for safari implement a delay prior to calling updateSubPage.
	if(navigator.userAgent.indexOf('AppleWebKit') >= 0){
        //delay 100 milliseconds before retrieving and displaying the sub-page
		setTimeout(updateSubPage, 100);
	}
	else{
        //retrieve and display the sub-page matching the command
		updateSubPage();
	}
    //a timing loop to check to see if a sub-page needs to be loaded and displayed
	setInterval(checkURLForUpdate, pageUpdateDelayMillis);
}
/*  
*   a function used in setInterval to check the URL and see if the sub-page needs to be changed. 
*/
function checkURLForUpdate(){
    //don't do anything if the page or the URL or the page is currently being updated
	if(!session.hasAttribute('updatingPage') && !session.hasAttribute('updatingURL')){
        //get the current anchor declaration in the URL
        var aCmd = getSubPageCommand();
        //if the sessions urlCmd attribute doesn't match the new one update the page
		if(session.getAttribute('urlCmd') != null && session.getAttribute('urlCmd') != aCmd){
			updateSubPage();
		}
	}
}

/* 
*  change the sub-page being displayed 
*/
function updateSubPage(){
	var pageCmd = getSubPageCommand();
	//add an attribute to block the automatic update checker from calling this function
	session.addAttribute('updatingPage', 'true');
    //add the current command to the session
	session.addAttribute('urlCmd', pageCmd);
    //tell the dispatcher to load the page
	theController.dispatch(pageCmd);
	//if the iframe technique is used in Firefox for windows each sub-page will appear twice so don't use it.
    if(navigator.userAgent.indexOf('Firefox') < 0){
		useHistoryIframe(pageCmd);
	}
    //free up this function for use by the automatic update checker
	session.removeAttribute('updatingPage');
}

/*
*  Change the anchor indicator in the URL
*/
function updateURL(aCmd, event){
    //lock this function so that it can't be called twice at the same time
	session.addAttribute('updatingURL', 'true');
	if(aCmd != null){
      //get the current command from the URL
      var currentCommand = getSubPageCommand();
      //replace the existing command with the new one if they are different
      if(currentCommand != aCmd){
		window.location.href = baseURL+'#'+aCmd;
	  }
      //make sure that the mouse event is retrieved
      event = event || window.event;
      //stop the mouse events standard action
      stopStandardBehavior(event);
    }
    //free up this function for use
	session.removeAttribute('updatingURL');
}
/*
*  retrieve the cmd from the location URL 
*/
function getSubPageCommand(){
    //retreive the command from the ULR location
	var urlArray = window.location.href.split('#');
    //set the default value if no command is found
	var pageCmd = 'home';
	if(urlArray.length > 1){
		pageCmd = urlArray[1];
	}
	return pageCmd;
}
 /*
 * stops the progression of an event 
 */
function stopStandardBehavior(event){
   /*
    * stop the event from actually following the default behavior
    */
    if(event){
        if (event.preventDefault)
                event.preventDefault();
        event.returnValue = false;
    }
}
/* 
*  updates the tracking iframe to contain the current command so it can be retrieved by Safari and IE
*/
function useHistoryIframe(aCmd){
	var histIframe = document.getElementById('history_frame');
	histIframe.src = 'tracking.html?'+aCmd;
}