var dropLocations = new Array();
function calculateDropLocations(aContainer, nodeType){
    //get all of the nodes that are the type nodeType in the content
    var anArray = aContainer.getElementsByTagName(nodeType);
    //get the length of the list so an invariant for loop can be used
    var anArrayLength = anArray.length;
    for(var i = 0; i < anArrayLength; i++){
        var aPoint = getUpperLeftPoint(anArray[i]);
        var aListLocation = new Object();
        aListLocation.point = aPoint;
        aListLocation.node = anArray[i];
        dropLocations[i] = aListLocation;
    }
    dropLocations.sort(sortLeftToRight);
}
function getDropLocation(x, y){
    var listLength = dropLocations.length;
    //examine all drop locations
    for(var i = 0; i < listLength; i++){
        var aDropLocation = dropLocations[i];
        //calculate the bounds of the potential drop location
        var left = aDropLocation.point.leftValue - oldScrollAmount.x;
        var right = left + aDropLocation.node.offsetWidth;
        var top = aDropLocation.point.topValue - oldScrollAmount.y;
        var bottom = top + aDropLocation.node.offsetHeight;
        //check if this location contains the drag event point
        if(left < x && right > x
            && top < y && bottom > y){
            
            return aDropLocation.node;
        }
    
    }
    return null;
}
function sortLeftToRight(location1, location2){
    return location1.point.leftValue - location2.point.leftValue;
}
function getUpperLeftPoint(aNode){
    var aPoint = new Object()
    aPoint.leftValue = aNode.offsetLeft;
    aPoint.topValue = aNode.offsetTop;
	  while((aNode = aNode.offsetParent) != null){
            aPoint.leftValue += aNode.offsetLeft;
            aPoint.topValue += aNode.offsetTop;
	  }
	  return aPoint;
}

//this function must be set in the html file as the function for the onscroll event or it will not be fired
function docScrolled(event){
    event = event || window.event;
    var curDraggable = document.body.draggable;
    if(document.body.draggable){
        var currentScrollAmountY = getScrollAmountTop();
        var currentScrollAmountX = getScrollAmountLeft();
        
        var scrollChangeX = currentScrollAmountX - oldScrollAmount.x
        var scrollChangeY = currentScrollAmountY - oldScrollAmount.y
        oldScrollAmount.x = currentScrollAmountX;
        oldScrollAmount.y = currentScrollAmountY;
        
        var curTop  = curDraggable.top + scrollChangeY;
        var curLeft = curDraggable.left + scrollChangeX;
        
        var dragTop = curDraggable.top + currentScrollAmountY;
        var dragLeft = curDraggable.left +currentScrollAmountX;
        curDraggable.top = curTop;
        curDraggable.left = curLeft;
        
        curDraggable.style.top = curTop+'px';
        curDraggable.style.left = curLeft+'px';
                 
        
        if(curDraggable.top.whenDragging){
            curDraggable.whenDragging(event, curDraggable);
        }   

    }
}
function buildDraggables(aContainer, nodeType, startDraggingFunc, draggingFunc, droppedFunc){
    //get all of the nodes that are the type nodeType in the content
    var anArray = aContainer.getElementsByTagName(nodeType);
    //get the length of the list so an invariant for loop can be used
    var anArrayLength = anArray.length;
    for(var i = 0; i < anArrayLength; i++){
        var aNode = anArray[i];
        makeDraggable(aNode);
        aNode.whenStartDragging = startDraggingFunc;
        aNode.whenDragging = draggingFunc;
        aNode.whenDropped = droppedFunc;
        aNode.onselectstart = function() {
            return false; 
        };
    }
    oldScrollAmount.x = getScrollAmountLeft();
    oldScrollAmount.y = getScrollAmountTop();
}
var oldScrollAmount = new Object();
oldScrollAmount.x = 0;
oldScrollAmount.y = 0;
function makeDraggable(aNode){
    if(aNode.style){//position may have been set to fixed or absolute
        //the following values may or may not have been set in the style
        aNode.origPosType = aNode.style.position = aNode.style.position || 'relative';
        aNode.style.top = aNode.style.top || "0px";
        aNode.origX = aNode.style.left = aNode.style.left || "0px";
        
        aNode.origY = aNode.style.top.replace("px", "")*1;//force it to be integer.  If this is not done the additions will be treated as appends
        aNode.origX = aNode.style.left.replace("px", "")*1;
    }
    else{//no styling set at all
        aNode.origPosType = aNode.style.position = 'relative';
        aNode.style.top = "0px";
        aNode.style.left = "0px";
        
        aNode.origY = 0;
        aNode.origX = 0;
    }
    
    aNode.onmouseclick = function(event){
        this.isDragging = false;
    }
    //the standard start dragging function
    aNode.onmousedown = function(event){
        stopStandardBehavior(event);
        event = event || window.event;//make sure that IE gets the event
    
        if(body.draggable){//if an old draggable is still around an error has happened.  reset it to its original location
            body.draggable.isDragging = false;
            body.draggable.style.top = aNode.origY+"px";
            body.draggable.style.left = aNode.origX+"px";
            
            body.draggable.oldMouseY = body.draggable.origY;
            body.draggable.oldMouseX = body.draggable.origX;
        }
        this.top = aNode.origY;
        this.left = aNode.origX;
        if(this.whenStartDragging){
            this.whenStartDragging(event, this);
        }
        this.isDragging = true;
        this.style.zIndex = 500;
        //initialize the previous location value to be the current
        this.oldMouseX = event.clientX;
        this.oldMouseY = event.clientY;
        body.draggable = this;
        
        oldScrollAmount.x = getScrollAmountLeft();
        oldScrollAmount.y = getScrollAmountTop();
    };
    //the standard stop dragging function
    aNode.onmouseup = function(event){
        stopStandardBehavior(event);
        if(this.parentNode.style){
            this.style.zIndex = this.parentNode.style.zIndex;//send back down
        }
        else{
            this.style.zIndex = 0;
        }
        this.isDragging = false;
        body.draggable.isDragging = false;
        body.draggable = null;
        
        aNode.origY = this.top;
        aNode.origX = this.left;
        
        if(this.whenDropped){
            this.whenDropped(event, this);
        }
    };
    //the standard dragging function
    aNode.onmousemove = function(event){
        if(this.isDragging){
            stopStandardBehavior(event);
            event = event || window.event;//make sure that IE gets the event
            var curY = event.clientY;
            var curX = event.clientX;
            if(!this.ignoreScrolling){
                curY += getScrollAmountTop() - oldScrollAmount.y;
                curX += getScrollAmountLeft() - oldScrollAmount.x;
            }
            this.yDiff = curY - this.oldMouseY;
            this.xDiff = curX - this.oldMouseX;
            this.top += this.yDiff;
            this.left += this.xDiff;
            
            
            this.style.top = this.top+'px';
            this.style.left = this.left+'px';         
            this.oldMouseX = event.clientX;
            this.oldMouseY = event.clientY;
            if(this.whenDragging){
                this.whenDragging(event, this);
            }   
        }
    };
    aNode.setLocation = function (x, y){
        this.dragging = true;
        this.oldMouseX = this.oldMouseX || 0;
        this.oldMouseY = this.oldMouseY || 0;
        var simulatedDragEvent = new Object();
        simulatedDragEvent.clientX = x;
        simulatedDragEvent.clientY = y;
        this.onmousemove(simulatedDragEvent);
        this.dragging = false;
    }
    
   var body = document.getElementsByTagName('body')[0];
   if(!body.onmousemove){
        body.onmousemove = function(event){//error handling for if mouse drags outside of draggable Element
            if(body.draggable && body.draggable.isDragging){
                body.draggable.onmousemove(event);
            }
        };
   }
}

function getViewableHeight(){
    var viewableHeight = null;
    if(window.innerHeight){
   		viewableHeight = window.innerHeight;
	}
	else if(document.documentElement && document.documentElement.clientHeight){
		viewableHeight = document.documentElement.clientHeight;
	}
	else if(document.body){
		viewableHeight = document.body.clientHeight;
	}
    return viewableHeight;
}
function getViewableWidth(){
    var viewableWidth = null;
    if(window.innerWidth){
        viewableWidth = window.innerHeight;
	}
	else if(document.documentElement && document.documentElement.clientWidth){
		viewableWidth = document.documentElement.clientWidth;
	}
	else if(document.body){
		viewableWidth = document.body.clientWidth;
	}
    return viewableWidth;
}
function getScrollAmountTop(){
    var amountOfScroll = null;
    if(document.documentElement && document.documentElement.scrollTop) {
        amountOfScroll = document.documentElement.scrollTop;
    }
    else if (document.body) {
        amountOfScroll = document.body.scrollTop;
    }
    return amountOfScroll;
 }
 function getScrollAmountLeft(){
    var amountOfScroll = null;
    if(document.documentElement && document.documentElement.scrollLeft) {
        amountOfScroll = document.documentElement.scrollLeft;
    }
    else if (document.body) {
        amountOfScroll = document.body.scrollLeft;
    }
    return amountOfScroll;
 }
 function stopStandardBehavior(event){
	   /*
		* stop the event from actually following the default behavior
		*/
	    if(!event)
		   event = window.event;
		if(event){
			if (event.preventDefault)
					event.preventDefault();
			event.returnValue = false;
		}
	}